package com.yidianling.course.widget

import android.annotation.SuppressLint
import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.util.Log
import android.view.View
import android.widget.FrameLayout
import android.widget.SeekBar
import androidx.annotation.DrawableRes
import androidx.annotation.Nullable
import androidx.annotation.RawRes
import com.bumptech.glide.Glide
import com.ydl.media.audio.AudioPlayer
import com.ydl.media.audio.OnPlayerEventListener
import com.ydl.media.audio.enums.PlayModeEnum
import com.ydl.media.audio.model.Music
import com.ydl.media.view.PlayTypeEnum
import com.ydl.media.view.PlayerFloatHelper
import com.yidianling.common.tools.RxNetTool
import com.yidianling.course.R
import kotlinx.android.synthetic.main.audio_play_view.view.*

/**
 * 音频播放UI
 */
class AudioPlayView(context: Context, attrs: AttributeSet?) :
    FrameLayout(context, attrs), OnPlayerEventListener {

    private var mSeekBarIsTouch = false
    private var mProgress = 0
    var mNonWifiTips = true

    /**
     * 切换前调用 -> 判断切换后 音频 是否符合业务条件
     * 切换后调用 -> 若 音频 失效,自动切换至下一个,此时仍需判断是否符合业务条件
     */
    var mListener: ((playPosition: Int) -> Unit)? = null

    init {
        AudioPlayer.get().addOnPlayEventListener(this)
        View.inflate(context, R.layout.audio_play_view, this)
        iv_bg.setOnClickListener {
            AudioPlayer.get().playPause()
        }
        iv_rewind.setOnClickListener {
            AudioPlayer.get().seekTo(position = AudioPlayer.get().currentPosition.minus(15000))
        }
        iv_fast_forward.setOnClickListener {
            AudioPlayer.get().seekTo(position = AudioPlayer.get().currentPosition.plus(15000))
        }
        iv_pre.setOnClickListener {
            mListener?.invoke(AudioPlayer.get().playPosition - 1)
        }
        iv_next.setOnClickListener {
            mListener?.invoke(AudioPlayer.get().playPosition + 1)
        }
        seekbar.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                if (fromUser) {
                    tv_start.text = getStringTime(progress)
                    mProgress = progress
                }
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
                mSeekBarIsTouch = true
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
                mSeekBarIsTouch = false
                //拖动seekbar时不进行以下操作
                if (!mSeekBarIsTouch) {
                    AudioPlayer.get().seekTo(-1, mProgress.toLong())
                }
            }

        })
    }

    fun setData(playPosition: Int, playList: ArrayList<Music>, isAuto: Boolean, from: Int) {
        if (playList.isEmpty()) return
        setAutoNext(isAuto)
        AudioPlayer.get().addPlayList(playList)
        if (isPlaying()) mNonWifiTips = false
        if (RxNetTool.isWifi(context)) {
            playAudio(from, playPosition, playList)
        } else {
            if (mNonWifiTips) {
                showNonWifiTips {
                    mNonWifiTips = false
                    playAudio(from, playPosition, playList)
                }
            } else {
                playAudio(from, playPosition, playList)
            }
        }
    }

    private fun showNonWifiTips(block: () -> Unit) {
        block.invoke()
    }

    private fun playAudio(from: Int, playPosition: Int, playList: ArrayList<Music>) {
        if (isPlaying() && (from == 1 || from == 2)) {
            if (TextUtils.equals(getCurrentUrl(), playList[playPosition].path)) {
                seekbar.max = AudioPlayer.get().getDuration().toInt()
                tv_duration.text = getStringTime(seekbar.max)
            } else {
                play(playPosition)
            }
        } else {
            play(playPosition)
        }
    }

    /**
     * 获取当前播放url
     */
    private fun getCurrentUrl(): String {
        return AudioPlayer.get().playMusic?.path.orEmpty()
    }

    fun play(index: Int) {
        PlayerFloatHelper.playingType = PlayTypeEnum.PLAY_TYPE_COURSE
        AudioPlayer.get().load(index)
    }

    /**
     * 设置自动播放下一曲
     */
    fun setAutoNext(auto: Boolean) {
        if (!auto) {
            AudioPlayer.get().playMode = PlayModeEnum.SINGLE
        } else {
            AudioPlayer.get().playMode = PlayModeEnum.LIST_LOOP
        }
    }

    @SuppressLint("SetTextI18n")
    override fun onLoad(music: Music) {
        displayPlayStatus(R.drawable.course_loading5, true)
        seekbar.progress = 0
        tv_start.text = "00:00"
    }

    override fun onPlayerStart() {
    }

    override fun onPrepared(duration: Long) {
        seekbar.max = duration.toInt()
        tv_duration.text = getStringTime(seekbar.max)
        mListener?.invoke(AudioPlayer.get().playPosition)
    }

    override fun onBufferingUpdate(percent: Int) {
        seekbar.secondaryProgress = percent * seekbar.max / 100
    }

    override fun onPublish(percent: Int, currentPosition: Long) {
        if (!mSeekBarIsTouch) {
            displayPlayStatus(R.drawable.course_pause)
            seekbar.progress = currentPosition.toInt()
            tv_start.text = getStringTime(currentPosition.toInt())
        }
    }

    override fun onComplete() {
    }

    override fun onPlayerPause() {
        if (isPlaying()) {
            displayPlayStatus(R.drawable.course_pause)
        } else {
            displayPlayStatus(R.drawable.course_play)
        }
    }

    private fun displayPlayStatus(
        @RawRes @DrawableRes @Nullable resourceId: Int,
        isGif: Boolean = false
    ) {
        context?.let {
            if (isGif) {
                Glide.with(it).asGif().load(resourceId).into(iv_play_status)
            } else {
                Glide.with(it).asBitmap().load(resourceId).into(iv_play_status)
            }
        }
    }

    private fun getStringTime(time: Int): String {
        if (time <= 0) return "00:00"
        val min = time / 60000
        val ss = (time - min * 60000) / 1000

        var m = "" + min
        var s = "" + ss
        if (min < 10) {
            m = "0$m"
        }
        if (ss < 10) {
            s = "0$ss"
        }
        return "$m:$s"
    }

    fun onDestroy() {
        AudioPlayer.get().removeOnPlayEventListener(this)
    }

    fun setSpeed(speed: Float) {
        AudioPlayer.get().setSpeed(speed)
    }

    fun isPlaying() = AudioPlayer.get().isPlaying

    fun pausePlay() = AudioPlayer.get().pausePlayer()

    fun getAudioMusic() = AudioPlayer.get().playMusic

}